@@ -21,11 +21,33 @@ use vars qw(@ISA) ;
 
 use XML::Smart::Tie ;
 use XML::Smart::Tree ;
-use XML::Smart::Data qw(data) ;
-use XML::Smart::XPath qw(xpath XPath xpath_pointer XPath_pointer) ;
 
 our ($VERSION) ;
-$VERSION = '1.5' ;
+$VERSION = '1.6' ;
+
+###############
+# AUTOLOADERS #
+###############
+
+sub data {
+  require XML::Smart::Data ;
+  *data = \&XML::Smart::Data::data ;
+  &XML::Smart::Data::data(@_) ;
+}
+
+sub xpath { _load_xpath() ; &XML::Smart::XPath::xpath(@_) ;}
+sub XPath { _load_xpath() ; &XML::Smart::XPath::XPath(@_) ;}
+sub xpath_pointer { _load_xpath() ; &XML::Smart::XPath::xpath_pointer(@_) ;}
+sub XPath_pointer { _load_xpath() ; &XML::Smart::XPath::XPath_pointer(@_) ;}
+
+sub _load_xpath {
+  require XML::Smart::XPath ;
+  *xpath = \&XML::Smart::XPath::xpath ;
+  *XPath = \&XML::Smart::XPath::XPath ;
+  *xpath_pointer = \&XML::Smart::XPath::xpath_pointer ;
+  *XPath_pointer = \&XML::Smart::XPath::XPath_pointer ;
+  *_load_xpath = sub {} ;
+}
 
 #################
 # NO_XML_PARSER #
@@ -42,7 +64,7 @@ sub NO_XML_PARSER {
 sub new {
   my $class = shift ;
   my $file = shift ;
-  my $parser = shift ;
+  my $parser = ($_[0] !~ /^(?:uper|low|arg|on)\w+$/i) ? shift(@_) : '' ;
   
   my $this = Object::MultiType->new(
   boolsub   => \&boolean ,
@@ -200,7 +222,7 @@ sub back {
   my $this = shift ;
   
   my @tree = @{$$this->{keyprev}} ;
-  if (!@tree) { return back ;}
+  if (!@tree) { return $this ;}
   
   my $last = pop(@tree) ;
   my $i = 0 ;
@@ -325,11 +347,15 @@ sub copy {
   $$copy->{tree} = &_copy_hash($this->tree) ;
   $$copy->{keyprev} = $$this->{keyprev} ;
   
+  bless($copy, ref($this)) ;
+  
   my ( $back , $key , $i ) = $copy->back ;
   
-  $copy = $back->{$key} ;
-  $copy = $back->[$i] if $i ;
-  
+  if ( $key ne '' ) {
+    $copy = $back->{$key} ;
+    $copy = $back->[$i] if $i ;
+  }
+    
   return( $copy ) ;
 }
 
@@ -368,6 +394,60 @@ sub _copy_hash {
   return( $copy ) ;
 }
 
+###########
+# TREE_OK #
+###########
+
+sub tree_ok {
+  return _tree_ok_parse( &tree ) ;
+}
+
+##############
+# POINTER_OK #
+##############
+
+sub pointer_ok {
+  return _tree_ok_parse( &pointer ) ;
+}
+
+sub tree_pointer_ok { &pointer_ok ;}
+
+##################
+# _TREE_OK_PARSE #
+##################
+
+sub _tree_ok_parse {
+  my ( $ref ) = @_ ;
+  my $copy ;
+  
+  if (ref $ref eq 'HASH') {
+    $copy = {} ;
+    foreach my $Key ( keys %$ref ) {
+      next if $Key eq '/order' || $Key eq '/nodes' || $Key =~ /\/\.CONTENT\// ;
+      if (ref $$ref{$Key}) {
+        $$copy{$Key} =&_tree_ok_parse($$ref{$Key}) ;
+      }
+      else { $$copy{$Key} = $$ref{$Key} ;}
+    }
+  }
+  elsif (ref $ref eq 'ARRAY') {
+    $copy = [] ;
+    foreach my $i ( @$ref ) {
+      if (ref $i) {
+        push(@$copy , &_tree_ok_parse($i) ) ;
+      }
+      else { push(@$copy , $i) ;}
+    }
+  }
+  elsif (ref $ref eq 'SCALAR') {
+    my $copy = $$ref ;
+    return( \$copy ) ;
+  }
+  else { return( {} ) ;}
+
+  return( $copy ) ;
+}
+
 ########
 # TREE #
 ########
@@ -375,6 +455,39 @@ sub _copy_hash {
 sub tree { return( ${$_[0]}->{tree} ) ;}
 sub tree_pointer { &pointer ;}
 
+#############
+# DUMP_TREE #
+#############
+
+sub dump_tree {
+  require Data::Dumper ;
+  return Data::Dumper::Dumper( &tree ) ;
+}
+
+sub dump_tree_ok {
+  require Data::Dumper ;
+  return Data::Dumper::Dumper( &tree_ok ) ;
+}
+
+
+################
+# DUMP_POINTER #
+################
+
+sub dump_pointer {
+  require Data::Dumper ;
+  return Data::Dumper::Dumper( &pointer ) ;
+}
+
+sub dump_pointer_ok {
+  require Data::Dumper ;
+  return Data::Dumper::Dumper( &pointer_ok ) ;
+}
+
+
+sub dump_tree_pointer { &dump_pointer ;}
+sub dump_tree_pointer_ok { &dump_pointer_ok ;}
+
 ###########
 # POINTER #
 ###########
@@ -519,7 +632,9 @@ sub set_node {
   my $nodes = $back->{'/nodes'}->pointer ;
   
   if ( $bool ) {
-    $$nodes{$key} = 1 ;
+    if ( $$nodes{$key} =~ /^(\w+,\d+),(\d*)/ ) { $$nodes{$key} = "$1,1" ;}
+    else { $$nodes{$key} = 1 ;}
+
     if ( !$this->{CONTENT} ) {
       my $content = $this->content ;
       $this->{CONTENT} = $content if $content ne '' ;
@@ -552,6 +667,139 @@ sub set_order {
   @{$$pointer{'/order'}} = @_ ;
 }
 
+sub order {
+  my $this = shift ;
+  my $pointer = $$this->{point} ;
+  return @{$$pointer{'/order'}} if defined $$pointer{'/order'} && ref($$pointer{'/order'}) eq 'ARRAY' ;
+  return() ;
+}
+
+#############
+# SET_CDATA #
+#############
+
+sub set_node_type {
+  my $this = shift ;
+  my ( $type , $bool ) = @_ ;
+  if ( $#_ < 1 ) { $bool = 1 ;}
+  
+  my $key = $this->key ;
+  
+  my $back = $this->back ;
+  
+  $back->{'/nodes'} = {} if $back->{'/nodes'}->null ;
+  my $nodes = $back->{'/nodes'}->pointer ;
+  
+  if ( $bool ) {
+    if ( $$nodes{$key} =~ /^\w+,\d+,(\d*)/ ) {
+      my $val = $1 ;
+      $$nodes{$key} = "$type,1,$val" ;
+    }
+    else { $$nodes{$key} = "$type,1,$$nodes{$key}" ;}
+
+    if ( !$this->{CONTENT} ) {
+      my $content = $this->content ;
+      $this->{CONTENT} = $content if $content ne '' ;
+    }
+  }
+  else {
+    if ( !$$nodes{$key} ) {
+      my $tp = _data_type( $back->{$key} ) ;
+      if ( $tp > 2 ) { $$nodes{$key} = "$type,0," ;}
+    }
+    elsif ( $$nodes{$key} eq '1' ) { $$nodes{$key} = "$type,0,1" ;}
+    elsif ( $$nodes{$key} =~ /^\w+,\d+,1/ ) { $$nodes{$key} = "$type,0,1" ;}
+    elsif ( $$nodes{$key} =~ /^\w+,\d+,0?$/ ) {
+      delete $$nodes{$key} ;
+      my @keys = keys %$this ;
+      if ( $#keys == 0 && @keys[0] eq 'CONTENT') {
+        my $content = $this->{CONTENT}('.') ;
+        $this->back->pointer->{$key} = $content ;
+      }
+    }
+  }
+}
+
+#############
+# SET_CDATA #
+#############
+
+sub set_cdata {
+  my $this = shift ;
+  $this->set_node_type('cdata',@_) ;
+}
+
+##############
+# SET_BINARY #
+##############
+
+sub set_binary {
+  my $this = shift ;
+  $this->set_node_type('binary',@_) ;
+}
+
+#################
+# SET_AUTO_NODE #
+#################
+
+sub set_auto_node {
+  my $this = shift ;
+  
+  my $key = $this->key ;
+  my $back = $this->back ;
+  
+  $back->{'/nodes'} = {} if $back->{'/nodes'}->null ;
+  my $nodes = $back->{'/nodes'}->pointer ;
+  
+  if ( !$$nodes{$key} || $$nodes{$key} eq '1' ) { ; }
+  elsif ( $$nodes{$key} =~ /^\w+,\d+,1/ ) { $$nodes{$key} = 1 ;}
+  elsif ( $$nodes{$key} =~ /^\w+,\d+,0?$/ ) {
+    delete $$nodes{$key} ;
+    my @keys = keys %$this ;
+    if ( $#keys == 0 && @keys[0] eq 'CONTENT') {
+      my $content = $this->{CONTENT}('.') ;
+      $this->back->pointer->{$key} = $content ;
+    }
+  }
+}
+
+############
+# SET_AUTO #
+############
+
+sub set_auto {
+  my $this = shift ;
+  
+  my $key = $this->key ;
+  my $back = $this->back ;
+  
+  $back->{'/nodes'} = {} if $back->{'/nodes'}->null ;
+  my $nodes = $back->{'/nodes'}->pointer ;
+  
+  delete $$nodes{$key} ;
+  my @keys = keys %$this ;
+  if ( $#keys == 0 && @keys[0] eq 'CONTENT') {
+    my $content = $this->{CONTENT}('.') ;
+    $this->back->pointer->{$key} = $content ;
+  }
+}
+
+##############
+# _DATA_TYPE #
+##############
+
+## 4 binary
+## 3 CDATA
+## 2 content
+## 1 value
+
+sub _data_type {
+  return 4 if ($_[0] =~ /[^\w\d\s!"#\$\%&'\(\)\*\+,\-\.\/:;<=>\?\@\[\\\]\^\`\{\|}~]/s) ;
+  return 3 if ($_[0] =~ /<.*?>/s) ;
+  return 2 if ($_[0] =~ /[\r\n\t]/s) ;
+  return 1 ;
+}
+
 #######
 # RET #
 #######
@@ -677,6 +925,7 @@ sub find_arg {
       my $data ;
       if ($name =~ /^content$/i) { $name = 'CONTENT' ;}
       $data = ref($hash) eq 'HASH' ? $$hash{$name} : $hash ;
+      $data = $$data{CONTENT} if ref($data) eq 'HASH' ;
       
       if    ($type eq 'eq'  && $data eq $value)     { push(@hash,$hash_i) ; push(@i,$i) ; last ;}
       elsif ($type eq 'ne'  && $data ne $value)     { push(@hash,$hash_i) ; push(@i,$i) ; last ;}
@@ -729,7 +978,7 @@ sub find_arg {
 
 sub content {
   my $this = shift ;
-  my $set_i = shift if $#_ > 0 ;
+  my $set_i = $#_ > 0 ? shift : undef ;
   
   if ( defined $$this->{content} ) {
     if (@_) { ${$$this->{content}} = $_[0] ;}
@@ -747,7 +996,7 @@ sub content {
     return $$this->{point}{$key}[$i] ;
   }
   elsif (exists $$this->{point}{$key}) {
-    if (@_ && ( my $tie = tied($$this->{point}{$key}) )) { $tie->STORE($set_i , $_[0]) ;}
+    if (@_ && ( my $tie = tied($$this->{point}{$key}) ) ) { $tie->STORE($set_i , $_[0]) ;}
     if ( wantarray && ( my $tie = tied($$this->{point}{$key}) ) ) { return $tie->FETCH(1) ;}
     return $$this->{point}{$key} ;
   }
@@ -826,7 +1075,7 @@ XML::Smart - A smart, easy and powerful way to access/create XML files/data.
 
 This module has an easy way to access/create XML data. It's based on the HASH
 tree that is made of the XML data, and enable a dynamic access to it with the
-Perl syntax for Hashe and Array, without needing to care if you have a Hashe or an
+Perl syntax for Hash and Array, without needing to care if you have a Hash or an
 Array in the tree. In other words, B<each point in the tree work as a Hash and
 an Array at the same time>!
 
@@ -973,7 +1222,7 @@ clone of XML::Parser::Lite with some fixes.
 
 =item OPTIONS
 
-You can force the uper case and lower case for tags (nodes) and arguments (attributes).
+You can force the uper case and lower case for tags (nodes) and arguments (attributes), and other extra things.
 
 =over 10
 
@@ -993,6 +1242,29 @@ Make the tags uper case.
 
 Make the arguments uper case.
 
+=item arg_single
+
+Set the value of arguments to 1 when they have a I<undef> value. 
+
+I<** This option will work only when the XML is parsed by B<XML::Smart::HTMLParser>, since it accept arguments without values:>
+
+  my $xml = new XML::Smart(
+  '<root><foo arg1="" flag></root>' ,
+  'XML::Smart::HTMLParser' ,
+  arg_single => 1 ,
+  ) ;
+
+In this example the option "arg_single" was used, what will define I<flag> to 1, but I<arg1> will still have a null string value ("").
+
+Here's the tree of the example above:
+
+  'root' => {
+              'foo' => {
+                         'flag' => 1,
+                         'arg1' => ''
+                       },
+            },
+
 =item on_start (CODE) I<*optional>
 
 Code/sub to call on start a tag.
@@ -1017,9 +1289,25 @@ I<** This options are applied when the XML data is loaded. For XML generation se
 
 =back
 
-Example of use:
+B<Examples of use:>
 
-  my $xml = XML::Smart->new($data , 'html' ,
+  my $xml_from_url = XML::Smart->new("http://www.perlmonks.org/index.pl?node_id=16046") ;
+  
+  ...
+  
+  my $xml_from_str = XML::Smart->new(q`<?xml version="1.0" encoding="iso-8859-1" ?>
+  <root>
+    <foo arg="xyz"/>
+  </root>
+  `) ;
+
+  ...
+
+  my $null_xml = XML::Smart->new() ;
+
+  ...
+
+  my $xml_from_html = XML::Smart->new($html_data , 'html' ,
   lowtag => 1 ,
   lowarg => 1 ,
   on_char => sub {
@@ -1251,6 +1539,17 @@ Make the tree from current point in the XML tree (not from the base as data()).
 
 Accept the same OPTIONS of the method B<I<data()>>.
 
+=head2  dump_tree()
+
+Dump the tree of the object using L<Data::Dumper>.
+
+=head2  dump_tree_pointer()
+
+Dump the tree of the object, from the pointer, using L<Data::Dumper>.
+
+=head2  dump_pointer()
+
+I<** Same as dump_tree_pointer()>.
 
 =head2  i()
 
@@ -1280,6 +1579,10 @@ Return the nodes names (not the object) in the pointer (keys that aren't argumen
 
 Return I<true> if the XML object has a null tree or if the pointer is in some place that doesn't exist.
 
+=head2  order()
+
+Return the order of the keys. See I<set_order()>.
+
 =head2  path()
 
 Return the path of the pointer.
@@ -1300,6 +1603,10 @@ Return the path of the pointer in the XPath format.
 
 Return the HASH tree from the pointer.
 
+=head2  pointer_ok
+
+Return a copy of the tree of the object, B<from the pointer>, but without internal keys added by I<XML::Smart>.
+
 =head2 root
 
 Return the ROOT name of the XML tree (main key).
@@ -1312,11 +1619,43 @@ Save the XML data inside a file.
 
 Accept the same OPTIONS of the method B<I<data()>>.
 
+=head2  set_auto
+
+Define the key to be handled automatically. Soo, data() will define automatically if it's a node, content or attribute.
+
+I<** This method is useful to remove set_node(), set_cdata() and set_binary() changes.>
+
+=head2  set_auto_node
+
+Define the key as a node, and data() will define automatically if it's CDATA or BINARY.
+
+I<** This method is useful to remove set_cdata() and set_binary() changes.>
+
+=head2  set_binary(BOOL)
+
+Define the node as a BINARY content when TRUE, or force to B<not> handle it as a BINARY on FALSE.
+
+Example of node handled as BINARY:
+
+  <root><foo dt:dt="binary.base64">PGgxPnRlc3QgAzwvaDE+</foo></root>
+
+Original content of foo (the base64 data):
+
+  <h1>test \x03</h1>
+
+=head2  set_cdata(BOOL)
+
+Define the node as CDATA when TRUE, or force to B<not> handle it as CDATA on FALSE.
+
+Example of CDATA node:
+
+  <root><foo><![CDATA[bla bla bla <tag> bla bla]]></foo></root>
+
 =head2  set_node(BOOL)
 
 Set/unset the current key as a node (tag).
 
-** If BOOL is not defined will use I<true>.
+** If BOOL is not defined will use I<TRUE>.
 
 =head2  set_order(KEYS)
 
@@ -1337,6 +1676,14 @@ object that works like a HASH/ARRAY through tie.
 
 Same as I<pointer()>.
 
+=head2  tree_ok()
+
+Return a copy of the tree of the object, but without internal keys added by I<XML::Smart>, like I</order> and I</nodes>.
+
+=head2  tree_pointer_ok()
+
+Return a copy of the tree of the object, B<from the pointer>, but without internal keys added by I<XML::Smart>.
+
 =head2  xpath() || XPath()
 
 Return a XML::XPath object, based in the XML root in the tree.
@@ -1526,6 +1873,8 @@ from the method:
 So, if you use the object as a string it works as a string,
 if you use as an object it works as an object! ;-P
 
+I<**See the method content() for more.>
+
 =head1 CREATING XML DATA
 
 To create XML data is easy, you just use as a normal HASH, but you don't need
@@ -1553,7 +1902,7 @@ to care with multiple nodes, and ARRAY creation/convertion!
   <server os="Linux" type="mandrake" version="8.9">
     <address>192.168.3.201</address>
     <address>192.168.3.202</address>
-  </server>ok 19
+  </server>
 
 After create your XML tree you just save it or get the data:
 
@@ -1661,7 +2010,7 @@ or if you don't want to care with quotes, end tags, etc... when writing by hand
 So, you can write by hand a bad XML file, load it with I<XML::Smart::HTMLParser>, and B<rewrite well>
 saving it again! ;-P
 
-** Note that <SCRIPT> tags will only by parse right if the content is inside
+** Note that <SCRIPT> tags will only parse right if the content is inside
 comments <!--...-->, since they can have tags:
 
   <SCRIPT LANGUAGE="JavaScript"><!--
@@ -1710,7 +2059,7 @@ an ARRAY. You also have other extra resources, like a search for nodes by attrib
 
 The idea for this module, came from the problem that exists to access a complex struture in XML.
 You just need to know how is this structure, something that is generally made looking the XML file (what is wrong).
-But in the same time is hard to always check (by code) the struture, before access it.
+But at the same time is hard to always check (by code) the struture, before access it.
 XML is a good and easy format to declare your data, but to extrac it in a tree way, at least in my opinion,
 isn't easy. To fix that, came to my mind a way to access the data with some query language, like SQL.
 The first idea was to access using something like:
@@ -1831,9 +2180,9 @@ key. So this actually returns another object, pointhing (inside it) to the key:
 
 =head1 TODO
 
-* Finish XPath implementation.
-* DTD.
-* implement a better way to declare meta tags.
+  * Finish XPath implementation.
+  * DTD.
+  * Implement a better way to declare meta tags.
 
 =head1 SEE ALSO
 
@@ -1852,13 +2201,16 @@ Graciliano M. P. <gm@virtuasites.com.br>
 
 I will appreciate any type of feedback (include your opinions and/or suggestions). ;-P
 
-Before make this module I dislike to use XML, and made everything to avoid it.
-Now I can use XML fine! ;-P
+Enjoy and thanks for who are enjoying this tool and have sent e-mails! ;-P
+
+=head1 THANKS
+
+Thanks to Rusty Allen for the extensive tests of CDATA and BINARY handling of XML::Smart.
+
+Thanks to Ted Haining to point a Perl-5.8.0 bug for tied keys of a HASH.
 
 Thanks to everybody that have sent ideas, patches or pointed bugs.
 
-Enjoy and thanks for who are enjoying this tool and have sent e-mails! ;-P
-
 =head1 COPYRIGHT
 
 This program is free software; you can redistribute it and/or
