@@ -24,6 +24,8 @@ our @EXPORT_OK = @EXPORT ;
 use strict ;
 no warnings ;
 
+use XML::Smart::Entity qw(_add_basic_entity) ;
+
 ########
 # DATA #
 ########
@@ -44,8 +46,10 @@ sub data {
     else {
       my $ks = keys %$tree ;
       my $n = 1 ;
-      if (ref $$tree{'/nodes'} eq 'HASH')  { ++$n if (keys %{$$tree{'/nodes'}}) ;}
-      if (ref $$tree{'/order'} eq 'ARRAY') { ++$n if @{$$tree{'/order'}} ;}
+      if (ref $$tree{'/nodes'} eq 'HASH')  { ++$n ;}
+      if (ref $$tree{'/order'} eq 'ARRAY') { ++$n ;}
+      #if (ref $$tree{'/nodes'} eq 'HASH')  { ++$n if (keys %{$$tree{'/nodes'}}) ;}
+      #if (ref $$tree{'/order'} eq 'ARRAY') { ++$n if @{$$tree{'/order'}} ;}
 
       if ($ks > $n) { $addroot = 1 ;}
       else {
@@ -77,7 +81,7 @@ sub data {
   my ($data,$unicode) ;
   {
     my $parsed = {} ;
-    &_data(\$data,$tree,'',-1, {} , $parsed , $args{noident} , $args{nospace} , $args{lowtag} , $args{lowarg} , $args{wild} , $args{sortall} ) ;
+    &_data(\$data , $tree , '' , -1 , {} , $parsed , undef , undef , $args{noident} , $args{nospace} , $args{lowtag} , $args{lowarg} , $args{wild} , $args{sortall} ) ;
     $data .= "\n" if !$args{nospace} ;
     if ( &_is_unicode($data) ) { $unicode = 1 ;}
   }
@@ -112,7 +116,7 @@ sub data {
     $meta = "\n" . join ("\n", @metas) ;
   }
   
-  my $wild = ' [format: wild]' if $args{wild} ;
+  my $wild = $args{wild} ? ' [format: wild]' : '' ;
   
   my $metagen = qq`\n<?meta name="GENERATOR" content="XML::Smart/$XML::Smart::VERSION$wild Perl/$] [$^O]" ?>` ;
   if ( $args{nometagen} ) { $metagen = '' ;}
@@ -185,7 +189,7 @@ sub _is_unicode {
 #########
 
 sub _data {
-  my ( $data , $tree , $tag , $level , $prev_tree , $parsed , @stat ) = @_ ;
+  my ( $data , $tree , $tag , $level , $prev_tree , $parsed , $ar_i , $node_type , @stat ) = @_ ;
 
   if (ref($tree) eq 'XML::Smart') { $tree = $$tree->{point} ;}
   
@@ -206,48 +210,63 @@ sub _data {
   if (ref($tree) eq 'HASH') {
     my ($args,$args_end,$tags,$cont,$stat_1) ;
     
-    my @all_keys ;
+    my (@all_keys , %multi_keys) ;
     
     if ( !$stat[5] && $tree->{'/order'} ) {
       my %keys ;
-      foreach my $keys_i ( @{$tree->{'/order'}} , sort keys %$tree ) {
-        if ( !$keys{$keys_i} && exists $$tree{$keys_i} ) {
+      foreach my $keys_i ( @{$tree->{'/order'}} ) {
+        if ( exists $$tree{$keys_i} && (!ref($$tree{$keys_i}) || ref($$tree{$keys_i}) eq 'HASH' || (ref($$tree{$keys_i}) eq 'ARRAY' && exists $$tree{$keys_i}[ $keys{$keys_i} ] ) ) ) {        
           push(@all_keys , $keys_i) ;
-          $keys{$keys_i} = 1 ;
+          
+          if ( ++$keys{$keys_i} == 2 && ref $$tree{$keys_i} eq 'ARRAY' ) {
+            my @val = map { ( $_ ne '' ? 1 : () ) } @{ $$tree{$keys_i} } ;
+            $multi_keys{$keys_i} = 1 if $#val > 0 ;
+          }
         }
       }
+      foreach my $keys_i ( sort keys %$tree ) {
+        if ( !$keys{$keys_i} && exists $$tree{$keys_i} ) { push(@all_keys , $keys_i) ;}
+      }
     }
     else { @all_keys = sort keys %$tree ;}
     
-    ##print "*** $tree->{'/order'} >> @all_keys\n" ;
-    
-    #if ( (defined $$tree{CONTENT} && $$tree{CONTENT} ne '') || (defined $$tree{content} && $$tree{content} ne '')) { $stat[0] = 1 ; $ident = '' ;}
-    
+    my %array_i ;
+
     foreach my $Key ( @all_keys ) {
-      #print "**>> $Key [$ident]\n" ;
       if ($Key eq '' || $Key eq '/order' || $Key eq '/nodes') { next ;}
-      if (ref($$tree{$Key})) {
+            
+      if ( $Key eq '!--' && (!ref($$tree{$Key}) || ( ref($$tree{$Key}) eq 'HASH' && (keys %{$$tree{$Key}}) == 1 && (defined $$tree{$Key}{CONTENT} || defined $$tree{$Key}{content}) ) ) ) {
+        my $ct = $$tree{$Key} ;
+        if (ref $$tree{$Key}) { $ct = defined $$tree{$Key}{CONTENT} ? $$tree{$Key}{CONTENT} : $$tree{$Key}{content} ;} ;
+        if ( $ct ne '' ) { $tags .= "$ident<!-- $ct -->" ;}
+      }
+      elsif (ref($$tree{$Key})) {
         my $k = $$tree{$Key} ;
+        my $i ;
         if (ref $k eq 'XML::Smart') { $k = ${$$tree{$Key}}->{point} ;}
-        $args .= &_data(\$tags,$k,$Key, $level+1 , $tree , $parsed , @stat) ;
+        elsif ( ref $k eq 'ARRAY' && $multi_keys{$Key} ) {
+          $i = $array_i{$Key}++ if $#{$k} > 0 ;
+        }
+        $args .= &_data(\$tags,$k,$Key, $level+1 , $tree , $parsed , $i , $$tree{'/nodes'}{$Key} , @stat) if $array_i{$Key} ne 'ok' ;
+        $array_i{$Key} = 'ok' if $i eq '' && ref $k eq 'ARRAY' ;
       }
       elsif ( $$tree{'/nodes'}{$Key} ) {
         my $k = [$$tree{$Key}] ;
-        $args .= &_data(\$tags,$k,$Key, $level+1 , $tree , $parsed , @stat) ;
+        $args .= &_data(\$tags,$k,$Key, $level+1 , $tree , $parsed , undef , $$tree{'/nodes'}{$Key} , @stat) ;
       }
       elsif ("\L$Key\E" eq 'content') {
         if ( tied($$tree{$Key}) && $$tree{$Key} =~ /\S/s ) {
           $ident = '' ; $stat[1] += 2 ;
         }
         next if tied($$tree{$Key}) ;
-        $cont .= $$tree{$Key} ;
+        
+        if ( $$tree{$Key} ne '' ) {
+          my $p0 = length($tags) ;
+          $tags .= $$tree{$Key} ;        
+          $cont = [$p0,length($tags)] ;
+        }
       }
       elsif ($Key =~ /^\/\.CONTENT\/\d+$/) { $tags .= $$tree{$Key} ;}
-      elsif ( $Key eq '!--' && (!ref($$tree{$Key}) || ( ref($$tree{$Key}) eq 'HASH' && keys %{$$tree{$Key}} == 1 && (defined $$tree{$Key}{CONTENT} || defined $$tree{$Key}{content}) ) ) ) {
-        my $ct = $$tree{$Key} ;
-        if (ref $$tree{$Key}) { $ct = defined $$tree{$Key}{CONTENT} ? $$tree{$Key}{CONTENT} : $$tree{$Key}{content} ;} ;
-        $cont .= '<!--' . $ct . '-->' ;
-      }
       elsif ( $stat[4] && $$tree{$Key} eq '') { $args_end .= " $Key" ;}
       else {
         my $tp = _data_type($$tree{$Key}) ;
@@ -277,26 +296,63 @@ sub _data {
       }
     }
     
-    &_add_basic_entity($cont) if $cont ne '' ;
+    foreach my $Key ( keys %array_i ) {
+      if ( $array_i{$Key} ne 'ok' && $#{ $$tree{$Key} } >= $array_i{$Key} ) {
+        for my $i ( $array_i{$Key} .. $#{ $$tree{$Key} } ) {
+          my $k = $$tree{$Key}[$i] ;
+          $args .= &_data(\$tags,$k,$Key, $level+1 , $tree , $parsed , undef , undef , @stat) ;
+        }
+      }
+    }
+    
+    if ( $cont ) {
+      my ( $po , $p1 ) = @$cont ;
+      my $cont = substr($tags , $po , $p1) ;
+        
+      my $tp = _data_type($cont) ;
+      
+      if ( $node_type =~ /^(\w+),(\d+),(\d*)$/ ) {
+        my ( $node_tp , $node_set ) = ($1,$2) ;
+
+        if ( !$node_set ) {
+          if    ( $tp == 3 && $node_tp eq 'cdata'  ) { $tp = 0 ;}
+          elsif ( $tp == 4 && $node_tp eq 'binary' ) { $tp = 0 ;}
+        }
+        else {
+          if    ( $node_tp eq 'cdata'  ) { $tp = 3 ;}
+          elsif ( $node_tp eq 'binary' ) { $tp = 4 ;}
+        }
+      }
+      
+      if ( $tp == 3 ) { $cont = "<![CDATA[$cont]]>" ;}
+      elsif ( $tp == 4 ) {
+        require XML::Smart::Base64 ;
+        $cont = &XML::Smart::Base64::encode_base64($cont) ;
+        $cont =~ s/\s$//s ;
+        $args .= ' dt:dt="binary.base64"' ;
+      }
+      else { &_add_basic_entity($cont) ;}
+      
+      my $px = $p1 ;
+      while( substr($tags , $px , 1) =~ /\s/ ) { ++$px ;}
+      
+      if ( $px > $p1 ) { substr($tags , $p1 , $px-$p1) = '' ;}
+      
+      substr($tags , $po , $p1) = $cont ;
+    }
     
     if ($args_end ne '') {
       $args .= $args_end ;
       $args_end = undef ;
     }
-    
-    #print "cont [$cont] $tags , $args , $cont\n" ;
-    
-    if ($args ne '' && $tags ne '') {
+
+    if (!@all_keys) {
+      $$data .= qq`$ident<$tag/>` if $tag ne '' ;
+    }
+    elsif ($args ne '' && $tags ne '') {
       $$data .= qq`$ident<$tag$args>` if $tag ne '' ;
       $$data .= $tags ;
-      $$data .= $cont ;
-      $$data .= $ident if $cont eq '' ;
-      $$data .= qq`</$tag>` if $tag ne '' ;
-    }
-    elsif ($args ne '' && $cont ne '') {
-      $$data .= qq`$ident<$tag$args>` if $tag ne '' ;
-      $$data .= $cont ;
-      $$data .= $ident if $cont eq '' ;
+      $$data .= $ident if !$cont ;
       $$data .= qq`</$tag>` if $tag ne '' ;
     }
     elsif ($args ne '') {
@@ -305,20 +361,21 @@ sub _data {
     elsif ($tags ne '') {
       $$data .= qq`$ident<$tag>` if $tag ne '' ;
       $$data .= $tags ;
-      $$data .= $cont ;
-      $$data .= $ident if $cont eq '' ;
-      $$data .= qq`</$tag>` if $tag ne '' ;
-    }
-    elsif ($cont ne '') {
-      $$data .= qq`$ident<$tag>` if $tag ne '' ;
-      $$data .= $cont ;
+      $$data .= $ident if !$cont ;
       $$data .= qq`</$tag>` if $tag ne '' ;
     }
   }
   elsif (ref($tree) eq 'ARRAY') {
     my ($c,$v,$tags) ;
-    
+
+    my $i = -1 ;
     foreach my $value_i (@$tree) {
+      
+      if ( $ar_i ne '' ) {
+        ++$i ;
+        next if $i != $ar_i ;
+      }
+      
       my $value = $value_i ;
       if (ref $value_i eq 'XML::Smart') { $value = $$value_i->{point} ;}
       
@@ -334,7 +391,7 @@ sub _data {
       elsif (ref($value)) {
         if (ref($value) eq 'HASH') {
           $c = 2 ;
-          &_data(\$tags,$value,$tag,$level, $tree , $parsed , @stat) ;
+          &_data(\$tags,$value,$tag,$level, $tree , $parsed , undef , undef , @stat) ;
           $do_val = 0 ;
         }
         elsif (ref($value) eq 'SCALAR') { $value = $$value ;}
@@ -342,6 +399,19 @@ sub _data {
       }
       if ( $do_val && $value ne '') {
         my $tp = _data_type($value) ;
+        
+        if ( $node_type =~ /^(\w+),(\d+),(\d*)$/ ) {
+          my ( $node_tp , $node_set ) = ($1,$2) ;
+          if ( !$node_set ) {
+            if    ( $tp == 3 && $node_tp eq 'cdata'  ) { $tp = 0 ;}
+            elsif ( $tp == 4 && $node_tp eq 'binary' ) { $tp = 0 ;}
+          }
+          else {
+            if    ( $node_tp eq 'cdata'  ) { $tp = 3 ;}
+            elsif ( $node_tp eq 'binary' ) { $tp = 4 ;}
+          }
+        }
+        
         if ($tp <= 2) {
           $c++ ;
           my $cont = $value ; &_add_basic_entity($value) ;
@@ -365,7 +435,7 @@ sub _data {
       }
     }
 
-    if ($c <= 1  && ! $$prev_tree{'/nodes'}{$tag}) {
+    if ( $ar_i eq '' && $c <= 1 && ! $$prev_tree{'/nodes'}{$tag}) {
       my $k = $stat[4] ? $tag : &_check_key($tag) ;
       if    ($stat[3] == 1) { $k = "\L$k\E" ;}
       elsif ($stat[3] == 2) { $k = "\U$k\E" ;}
@@ -397,12 +467,12 @@ sub _data {
 # _DATA_TYPE #
 ##############
 
-sub _data_type {
-  return 4 if ($_[0] =~ /[^\w\d\s!"#\$\%&'\(\)\*\+,\-\.\/:;<=>\?\@\[\\\]\^\`\{\|}~]/s) ;
-  return 3 if ($_[0] =~ /<.*?>/s) ;
-  return 2 if ($_[0] =~ /[\r\n\t]/s) ;
-  return 1 ;
-}
+## 4 binary
+## 3 CDATA
+## 2 content
+## 1 value
+
+sub _data_type { &XML::Smart::_data_type ;}
 
 ##############
 # _CHECK_TAG #
@@ -424,43 +494,6 @@ sub _check_key {
   return( $_[0] ) ;
 }
 
-#######################
-# _PARSE_BASIC_ENTITY #
-#######################
-
-sub _parse_basic_entity {
-  $_[0] =~ s/&lt;/</gs ;
-  $_[0] =~ s/&gt;/>/gs ;
-  $_[0] =~ s/&amp;/&/gs ;
-  $_[0] =~ s/&apos;/'/gs ;
-  $_[0] =~ s/&quot;/"/gs ;
-  
-  $_[0] =~ s/&#(\d+);/ $1 > 255 ? pack("U",$1) : pack("C",$1)/egs ;
-  $_[0] =~ s/&#x([a-fA-F\d]+);/pack("U",hex($1))/egs ;
-  
-  return( $_[0] ) ;
-}
-
-#####################
-# _ADD_BASIC_ENTITY #
-#####################
-
-sub _add_basic_entity {
-  $_[0] =~ s/(&(?:\w+;)?)/{_is_amp($1) or $1}/sgex ;
-  $_[0] =~ s/</&lt;/gs ;
-  $_[0] =~ s/>/&gt;/gs ;
-  return( $_[0] ) ;
-}
-
-###########
-# _IS_AMP #
-###########
-
-sub _is_amp {
-  if ($_[0] eq '&') { return( '&amp;' ) ;}
-  return( undef ) ;
-}
-
 ##############
 # _ADD_QUOTE #
 ##############
@@ -471,8 +504,8 @@ sub _add_quote {
   
   &_add_basic_entity($data) ;
   
-  my $q1 = 1 if $data =~ /"/s ;
-  my $q2 = 1 if $data =~ /'/s ;
+  my $q1 = ($data =~ /"/s) ? 1 : undef ;
+  my $q2 = ($data =~ /'/s) ? 1 : undef ;
   
   if (!$q1 && !$q2) { return( qq`"$data"` ) ;}
   
