@@ -3,7 +3,7 @@
 ###use Data::Dumper ; print Dumper( $XML->tree ) ;
 
 use Test;
-BEGIN { plan tests => 96 } ;
+BEGIN { plan tests => 133 } ;
 use XML::Smart ;
 
 no warnings ;
@@ -22,6 +22,46 @@ my $DATA = q`<?xml version="1.0" encoding="iso-8859-1"?>
     <server address="192.168.3.30" os="bsd" type="freebsd" version="9.0"/>
 </hosts>
 `;
+
+if(1){
+#########################
+{
+
+  my $XML = XML::Smart->new( q`
+  <html>
+    <head>
+      <title>Blah blah</title>
+    </head>
+    <body>
+      <form>
+        <input id="0"/>        
+        <br/>
+        <input id="2"/>
+        <br/>            
+      </form>
+    </body>
+    <null/>
+  </html>
+  ` );
+  
+  my $data = $XML->data( noheader => 1 ) ;
+  $data =~ s/\s+/ /gs ;
+  
+  ok($data , q`<html> <head> <title>Blah blah</title> </head> <body> <form> <input id="0"/> <br/> <input id="2"/> <br/> </form> </body> <null/> </html> `) ;
+  
+  my @order = $XML->{html}{body}{form}->order ;
+  ok(join(" ", @order) eq 'input br input br') ;
+  
+  $XML->{html}{body}{form}->set_order( qw(br input input br) ) ;
+  @order = $XML->{html}{body}{form}->order ;
+  ok(join(" ", @order) eq 'br input input br') ;
+
+  my $data = $XML->data( noheader => 1 ) ;
+  $data =~ s/\s+/ /gs ;
+  
+  ok($data , q`<html> <head> <title>Blah blah</title> </head> <body> <form> <br/> <input id="0"/> <input id="2"/> <br/> </form> </body> <null/> </html> `) ;
+
+}
 #########################
 {
 
@@ -96,14 +136,46 @@ content2
   
   my $data = $XML->data(noheader => 1) ;
   
-  ok($data , q`<root>
-  <tag1 arg1="123">
+  ok($data , q`<root>123<tag1 arg1="123">
     <sub arg="1">sub_content</sub>
   </tag1>
-  <tag2 arg1="123"/>123</root>
+  <tag2 arg1="123"/></root>
 
 `) ;
   
+}
+
+}
+#########################
+{
+
+  my $xml = new XML::Smart(q`
+<foo>
+TEXT1 & more
+<if.1>
+  aaa
+</if.1>
+<!-- CMT -->
+<elsif.2>
+  bbb
+</elsif.2>
+</foo>  
+  `,'html') ;
+  
+  my $data = $xml->data(noident=>1 , noheader => 1 , wild=>1) ;
+  
+  ok($data,q`<foo>
+TEXT1 &amp; more
+<if.1>
+  aaa
+</if.1>
+<!--  CMT  -->
+<elsif.2>
+  bbb
+</elsif.2></foo>
+
+`) ;
+
 }
 #########################
 {
@@ -158,7 +230,7 @@ content2
     <SCRIPT LANGUAGE="JavaScript"><!--
     function stopError() { return true; }
     window.onerror = stopError;
-    document.writeln("some <tag> wirtten!");
+    document.writeln("some >> written!");
     --></SCRIPT>
     <foo2 bar2=x2>
   </body></html>
@@ -167,7 +239,7 @@ content2
   my $data = $XML->data(noheader => 1 , nospace => 1) ;
   $data =~ s/\s//gs ;
   
-  ok($data,q`<html><title>TITLE</title><bodybgcolor="#000000"><foo1bar1="x1"/><SCRIPTLANGUAGE="JavaScript"><_-->functionstopError(){returntrue;}window.onerror=stopError;document.writeln("some&lt;tag&gt;wirtten!");</_--></SCRIPT><foo2bar2="x2"/></body></html>`);
+  ok($data,q`<html><title>TITLE</title><bodybgcolor="#000000"><foo1bar1="x1"/><SCRIPTLANGUAGE="JavaScript"><!--functionstopError(){returntrue;}window.onerror=stopError;document.writeln("some>>written!");--></SCRIPT><foo2bar2="x2"/></body></html>`);
 
 }
 #########################
@@ -206,13 +278,14 @@ content2
   
   my $XML = XML::Smart->new() ;
   
-  $XML->{sub}{sub2} = $XML0 ;
-  push(@{$XML->{sub}{sub2}} , $XML1 ) ;
+  $XML->{sub}{sub2} = $XML0->tree ;
+  push(@{$XML->{sub}{sub2}} , $XML1->tree ) ;
   
   my $data = $XML->data(noheader => 1) ;
   
   $data =~ s/\s//gs ;
   ok($data,'<sub><sub2><root><foo1name="x"/></root></sub2><sub2><root><foo2name="y"/></root></sub2></sub>') ;
+
 }
 #########################
 {
@@ -333,6 +406,7 @@ content2
     
   my $dataok = q`<serveros="Linux"type="mandrake"version="8.9"><address>192.168.3.201</address><address>192.168.3.202</address></server>`;
   ok($data,$dataok) ;
+
 }
 #########################
 {
@@ -341,7 +415,7 @@ content2
   my $data = $XML->data(noheader => 1) ;
   $data =~ s/\s//gs ;
   
-  my $dataok = qq`<fooport="80"><i>a</i><i>b</i>ct</foo>` ;
+  my $dataok = qq`<fooport="80">ct<i>a</i><i>b</i></foo>` ;
   
   ok($data,$dataok) ;
 
@@ -366,6 +440,7 @@ content2
   
   my $dataok = q`<HOSTS><SERVEROS="linux"TYPE="redhat"VERSION="8.0"><ADDRESS>192.168.0.1</ADDRESS><ADDRESS>192.168.0.2</ADDRESS></SERVER><SERVEROS="linux"TYPE="suse"VERSION="7.0"><ADDRESS>192.168.1.10</ADDRESS><ADDRESS>192.168.1.20</ADDRESS></SERVER><SERVERADDRESS="192.168.2.100"OS="linux"TYPE="conectiva"VERSION="9.0"/><SERVERADDRESS="192.168.3.30"OS="bsd"TYPE="freebsd"VERSION="9.0"/></HOSTS>`;
   ok($data,$dataok) ;
+  
 }
 #########################
 {
@@ -402,7 +477,7 @@ content2
   
   ok($data,$dataok) ;
                        
-  $XML->{hosts}[1]{server}[1] = {
+  $XML->{hosts}[1]{server}[0] = {
   os => 'LX'  ,
   type => 'red'  ,
   ver => 123 ,
@@ -414,15 +489,14 @@ content2
   $dataok = q`<root><hosts><serveros="lx"type="red"ver="123"/></hosts><hosts><serveros="LX"type="red"ver="123"/></hosts></root>`;
   
   ok($data,$dataok) ;
-  
-}
 
+}
 #########################
 {
 
   my $XML = XML::Smart->new('' , 'XML::Smart::Parser') ;
                           
-  $XML->{hosts}[1]{server}[1] = {
+  $XML->{hosts}[1]{server}[0] = {
   os => 'LX'  ,
   type => 'red'  ,
   ver => 123 ,
@@ -434,7 +508,7 @@ content2
   $dataok = q`<hosts><serveros="LX"type="red"ver="123"/></hosts>`;
   
   ok($data,$dataok) ;
-  
+
 }
 #########################
 {
@@ -448,12 +522,15 @@ content2
   } ;
 
   push( @{$XML->{hosts}} , {XXXXXX => 1}) ;
-  unshift( @{$XML->{hosts}{x}}  , $srv) ;
+  
+  unshift( @{$XML->{hosts}}  , $srv) ;
+  
+  push( @{$XML->{hosts}{more}}  , {YYYY => 1}) ;
   
   my $data = $XML->data(noheader => 1) ;
   $data =~ s/\s//gs ;
   
-  $dataok = q`<root><hostsos="lx"type="red"ver="123"/><hostsXXXXXX="1"/></root>` ;
+  $dataok = q`<root><hostsos="lx"type="red"ver="123"><moreYYYY="1"/></hosts><hostsXXXXXX="1"/></root>` ;
   
   ok($data,$dataok) ;
 
@@ -552,16 +629,16 @@ content2
 
   my $data = q`
   <root>
-    <foo bar="x"> My Company &amp; Name + &lt;tag&gt; &quot; + &apos;...</foo>
+    <foo bar="x"> My Company &amp; Name + x &gt;&gt; plus &quot; + &apos;...</foo>
   </root>
   `;
 
   my $XML = XML::Smart->new($data , 'XML::Smart::Parser') ;
   
-  ok($XML->{root}{foo} , q` My Company & Name + <tag> " + '...`) ;
+  ok($XML->{root}{foo} , q` My Company & Name + x >> plus " + '...`) ;
   
   my $data = $XML->data(nospace => 1 , noheader => 1 ) ;
-  ok($data , q`<root><foo bar="x"> My Company &amp; Name + &lt;tag&gt; " + '...</foo></root>`) ;
+  ok($data , q`<root><foo bar="x"> My Company &amp; Name + x &gt;&gt; plus " + '...</foo></root>`) ;
 
 }
 #########################
@@ -603,7 +680,7 @@ content2
   my $XML = XML::Smart->new($data , 'XML::Smart::Parser') ;
   
   ok( $XML->{root}{item}{data} , q`some CDATA code <non> <parsed> <tag> end`) ;
-
+  
 }
 #########################
 {
@@ -675,7 +752,6 @@ content2
   $XML->{menu}{arg2}->set_node ;
   my $data = $XML->data(nospace => 1 , noheader => 1 ) ;
   ok($data , q`<menu arg1="123"><arg2>456</arg2></menu>`) ;
-  
 
   $XML->{menu}{arg2}->set_node(0) ;
   my $data = $XML->data(nospace => 1 , noheader => 1 ) ;
@@ -690,6 +766,134 @@ content2
   my $data = $XML->data(nospace => 1 , noheader => 1 ) ;
   ok($data , q`<menu arg1="123"/>`) ;
 
+}
+#########################
+{
+
+
+  my $XML = XML::Smart->new() ;
+  $XML->{root}{foo} = "bla bla bla";
+
+  $XML->{root}{foo}->set_node(1) ;
+
+  ok( $XML->tree->{root}{'/nodes'}{foo} , '1' ) ;
+  ok( $XML->tree->{root}{foo}{CONTENT} , "bla bla bla" ) ;  
+
+  $XML->{root}{foo}->set_node(0) ;
+
+  ok( !exists $XML->tree->{root}{foo}{CONTENT} ) ;
+  ok( !exists $XML->tree->{root}{'/nodes'}{foo} ) ;
+  
+  $XML->{root}{foo}->set_cdata(1) ;
+  
+  ok( $XML->tree->{root}{'/nodes'}{foo} , 'cdata,1,' ) ;
+  ok( $XML->tree->{root}{foo}{CONTENT} , "bla bla bla" ) ;  
+  
+  $XML->{root}{foo}->set_node(1) ;
+  
+  ok( $XML->tree->{root}{'/nodes'}{foo} , 'cdata,1,1' ) ;
+  ok( $XML->tree->{root}{foo}{CONTENT} , "bla bla bla" ) ;  
+  
+  $XML->{root}{foo}->set_binary(1) ;
+  
+  ok( $XML->tree->{root}{'/nodes'}{foo} , 'binary,1,1' ) ;
+  ok( $XML->tree->{root}{foo}{CONTENT} , "bla bla bla" ) ;  
+  
+  $XML->{root}{foo}->set_binary(0) ;
+
+  ok( $XML->tree->{root}{'/nodes'}{foo} , 'binary,0,1' ) ;
+  ok( $XML->tree->{root}{foo}{CONTENT} , "bla bla bla" ) ;  
+  
+  $XML->{root}{foo}->set_auto_node ;
+  
+  ok( $XML->tree->{root}{'/nodes'}{foo} , 1 ) ;
+  ok( $XML->tree->{root}{foo}{CONTENT} , "bla bla bla" ) ;  
+  
+  $XML->{root}{foo}->set_cdata(0) ;
+  
+  ok( $XML->tree->{root}{'/nodes'}{foo} , 'cdata,0,1' ) ;
+  ok( $XML->tree->{root}{foo}{CONTENT} , "bla bla bla" ) ;
+  
+  $XML->{root}{foo}->set_binary(0) ;
+  
+  ok( $XML->tree->{root}{'/nodes'}{foo} , 'binary,0,1' ) ;
+  ok( $XML->tree->{root}{foo}{CONTENT} , "bla bla bla" ) ;
+  
+  $XML->{root}{foo}->set_auto ;
+  
+  ok( !exists $XML->tree->{root}{foo}{CONTENT} ) ;
+  ok( !exists $XML->tree->{root}{'/nodes'}{foo} ) ;
+
+}
+#########################
+{
+
+  my $XML = new XML::Smart $data1;
+  $XML->{root}{foo} = "bla bla bla <tag> bla bla";
+
+  my $data = $XML->data(nospace => 1 , noheader => 1 ) ;
+  ok($data , '<root><foo><![CDATA[bla bla bla <tag> bla bla]]></foo></root>') ;
+
+  $XML->{root}{foo}->set_cdata(0) ;
+  
+  $data = $XML->data(nospace => 1 , noheader => 1 ) ;
+  ok($data , '<root><foo>bla bla bla &lt;tag&gt; bla bla</foo></root>') ;
+  
+  $XML->{root}{foo}->set_binary(1) ;
+  
+  $data = $XML->data(nospace => 1 , noheader => 1 ) ;
+  ok($data , '<root><foo dt:dt="binary.base64">YmxhIGJsYSBibGEgPHRhZz4gYmxhIGJsYQ==</foo></root>') ;
+
+}
+#########################
+{
+
+  my $XML = new XML::Smart $data1;
+  $XML->{root}{foo} = "<h1>test \x03</h1>";
+
+  my $data = $XML->data(nospace => 1 , noheader => 1 ) ;
+  ok($data , '<root><foo dt:dt="binary.base64">PGgxPnRlc3QgAzwvaDE+</foo></root>') ;
+
+  $XML->{root}{foo}->set_binary(0) ;
+  
+  $data = $XML->data(nospace => 1 , noheader => 1 ) ;
+  ok($data , "<root><foo>&lt;h1&gt;test \x03\&lt;/h1&gt;</foo></root>") ;
+  
+  $XML->{root}{foo}->set_binary(1) ;
+  
+  $data = $XML->data(nospace => 1 , noheader => 1 ) ;
+  ok($data , '<root><foo dt:dt="binary.base64">PGgxPnRlc3QgAzwvaDE+</foo></root>') ;
+
+}
+#########################
+{
+
+  my $XML = new XML::Smart $data1;
+  $XML->{root}{foo} = "simple";
+
+  my $data = $XML->data(nospace => 1 , noheader => 1 ) ;
+  ok($data , '<root foo="simple"/>') ;
+  
+  $XML->{root}{foo}->set_cdata(1) ;
+
+  my $data = $XML->data(nospace => 1 , noheader => 1 ) ;
+  ok($data , '<root><foo><![CDATA[simple]]></foo></root>') ;
+  
+}
+#########################
+{
+
+  my $XML = new XML::Smart $data1;
+  $XML->{root}{foo} = "<words>foo bar baz</words>";
+
+  my $data = $XML->data(nospace => 1 , noheader => 1 ) ;
+  ok($data , '<root><foo><![CDATA[<words>foo bar baz</words>]]></foo></root>') ;
+  
+  $XML->{root}{foo}->set_cdata(0) ;
+
+  my $data = $XML->data(nospace => 1 , noheader => 1 ) ;
+  ok($data , '<root><foo>&lt;words&gt;foo bar baz&lt;/words&gt;</foo></root>') ;  
+
 }
 #########################
 {
@@ -726,6 +930,35 @@ content2
   ok($type->path , '/hosts/server[2]/type') ;
   ok($type->path_as_xpath , '/hosts/server[3]/@type') ;
     
+}
+#########################
+{
+
+  my $XML = new XML::Smart(q`
+  <root>
+    <output name='123'>
+      <frames format='a'/>
+      <frames format='b'/>
+    </output>
+    <output>
+      <name>456</name>
+      <frames format='c'/>
+      <frames format='d'/>
+    </output>
+  </root>
+  `,'smart');
+  
+  $XML = $XML->cut_root ;
+  
+  my @frames_123 = @{ $XML->{output}('name','eq',123){frames} } ;
+  my @formats_123 = map { $_->{format} } @frames_123 ;
+  
+  my @frames_456 = @{ $XML->{output}('name','eq',456){frames} } ;
+  my @formats_456 = map { $_->{format} } @frames_456 ;
+
+  ok( join(";", @formats_123) , 'a;b' ) ;
+  ok( join(";", @formats_456) , 'c;d' ) ;
+
 }
 #########################
 {
